//---------------------------------------------------------------------------


#pragma hdrstop

#include "History.h"
#include <assert.h>
#include <algorithm>
#include <fstream> 
#include <json/json.h>

//---------------------------------------------------------------------------

#pragma package(smart_init)


void History::AddEntry(History::Entry& entry)
{
	std::deque<Entry>::iterator iter;
	for (iter = entries.begin(); iter != entries.end(); ++iter)
	{
		if (*iter == entry)
		{
			entries.erase(iter);
			break;
		}
	}
	entries.push_front(entry);
	if (entries.size() > CALL_HISTORY_LIMIT)
	{
		entries.pop_back();
	}
	notifyObservers();	
}

int History::Read(void)
{
	assert(filename != "");
	Json::Value root;   // will contains the root value after parsing.
	Json::Reader reader;

	try
	{
		std::ifstream ifs(filename.c_str());
		std::string strConfig((std::istreambuf_iterator<char>(ifs)), std::istreambuf_iterator<char>());
		ifs.close();
		bool parsingSuccessful = reader.parse( strConfig, root );
		if ( !parsingSuccessful )
		{
			return 2;
		}
	}
	catch(...)
	{
		return 1;
	}

	const Json::Value &callHistory = root["CallHistory"];
	entries.clear();
	for (int i=0; i<std::min((unsigned int)CALL_HISTORY_LIMIT, callHistory.size()); i++)
	{
		const Json::Value &call = callHistory[i];
		struct Entry entry;

		entry.uri = call.get("uri", "").asString();
		entry.incoming = call.get("incoming", "").asBool();
		entry.time = call.get("time", 0).asInt();

		const Json::Value &ts = call["timestamp"];
		entry.timestamp.year = ts.get("year", 0).asInt();
		entry.timestamp.month = ts.get("month", 0).asInt();
		entry.timestamp.day = ts.get("day", 0).asInt();
		entry.timestamp.hour = ts.get("hour", 0).asInt();
		entry.timestamp.min = ts.get("min", 0).asInt();
		entry.timestamp.sec = ts.get("sec", 0).asInt();
		entry.timestamp.msec = ts.get("msec", 0).asInt();

		entries.push_back(entry);
	}

	notifyObservers();

	return 0;
}

int History::Write(void)
{
	assert(filename != "");
    Json::Value root;
	Json::StyledWriter writer;
	for (unsigned int i=0; i<entries.size(); i++)
	{
		Entry &entry = entries[i];
		root["CallHistory"][i]["uri"] = entry.uri;
		root["CallHistory"][i]["incoming"] = entry.incoming;
		root["CallHistory"][i]["time"] = entry.time;
		root["CallHistory"][i]["timestamp"]["year"] = entry.timestamp.year;
		root["CallHistory"][i]["timestamp"]["month"] = entry.timestamp.month;
		root["CallHistory"][i]["timestamp"]["day"] = entry.timestamp.day;
		root["CallHistory"][i]["timestamp"]["hour"] = entry.timestamp.hour;
		root["CallHistory"][i]["timestamp"]["min"] = entry.timestamp.min;
		root["CallHistory"][i]["timestamp"]["sec"] = entry.timestamp.sec;
		root["CallHistory"][i]["timestamp"]["msec"] = entry.timestamp.msec;
	}

	std::string outputConfig = writer.write( root );

	try
	{
		std::ofstream ofs(filename.c_str());
		ofs << outputConfig;
		ofs.close();
	}
	catch(...)
	{
    	return 1;
	}
		
	return 0;
}


